@@ -36,9 +36,8 @@
 #' @examples
 #' list_census_regions('CA16')
 list_census_regions <- function(dataset, use_cache = TRUE, quiet = FALSE) {
-  dataset = toupper(dataset)
   cache_file <- file.path(tempdir(),paste0(dataset, "_regions.rda"))
-  #cache_file <- cache_path(dataset, "_regions.rda")
+
   if (!use_cache || !file.exists(cache_file)) {
     if (!quiet) message("Querying CensusMapper API for regions data...")
     response <- httr::GET(paste0("https://censusmapper.ca/data_sets/", dataset,
